==> 1.0.2.tar.gz: lib/redux_reducer.dart <==
/// Redux Enhancement aims to provide an opiniotated approach for complex redux applications
///
/// For every class that is used within the state, a [ReduxReducer] is created.
/// These classes can be indefinitely nested, as long every Action (classes that extend [ReduxAction]) has for every
/// possible signature only ONE reducer.
/// Having multiple reducers will not produce errors, but the first that is found wins and that will be bad
///
/// ```
/// class Customer {
///   String name = '';
/// }
///
/// class AppState {
///   Customer customer = Customer();
/// }
///
/// class UpdateCustomerNameAction extends ReduxAction<String> {
///   UpdateCustomerNameAction(String value) : super(value);
/// }
///
/// class CustomerReducer extends ReduxReducer<AppState, Customer> {
///   CustomerReducer()
///       : super(
///     getter: (appStateBuilder) => appStateBuilder.customer,
///     setter: (appStateBuilder, customer) => appStateBuilder..customer = customer,
///     reducers: [
///       ActionReducer<Customer, UpdateCustomerNameAction, String>(_OnUpdateCustomerNameAction),
///     ],
///     children: [],
///   );
///
///   static Customer _OnUpdateCustomerNameAction(Customer customer, String name) => customer..name = name;
/// }
///
/// class AppStateReducer extends ReduxReducer<AppState, AppState> {
///   AppStateReducer()
///       : super(
///     getter: (state) => state,
///     setter: (state, value) => value,
///     reducers: [],
///     children: [
///       CustomerReducer(),
///     ],
///   );
///
///   static AppState _OnInitializeAction(AppState appState, _) => appState..initialized = true;
/// }
///
/// ```
///
///
library redux_enhancement;

import 'package:redux_enhancement/action_reducer.dart';
import 'package:redux_enhancement/redux_action.dart';
import 'package:redux_enhancement/redux_types.dart';

/// Every Reducer has to extend this class
/// ```
/// class ShopReducer extends ReduxReducer<AppState, Shop> {
///   static Map<Type, ReducerFunction<AppState, dynamic>> _reducers = {
///     UpdateShopNameAction: (shop, action) => shop..name = action.value,
///   }
///
///   ShopReducer()
///       : super(
///     getter: (appState) => appState.shop,
///     setter: (appState, value) => appState..shop = value,
///     reducers: _reducers,
///     children: [
///       AddressReducer<Shop>(
///             (shop) => shop.address,
///             (shop, address) => shop..address = address,
///       ),
///     ],
///   );
/// }
/// ```
///
abstract class ReduxReducer<Parent, ClassType> {
  GetterFunction<Parent, ClassType> getter;

  SetterFunction<Parent, ClassType> setter;

  List<ReduxReducer<ClassType, dynamic>> children;

  List<ActionReducer<ClassType, dynamic, dynamic>> reducers;

  ReduxReducer({
    required this.getter,
    required this.setter,
    required this.reducers,
    this.children = const [],
  });

  /// Retrieves the fitting reducer for the [ReduxAction] supplied, executes it and returns the [Parent]
  /// or null if no [ReducerFunction] was found in its [reducers] or within its [children]
  /// The first [ReducerFunction] that is found is used and the result returned immediately.
  /// You should never have multiple [ReducerFunction]s defined for any [ReduxAction]!
  Parent? reduce(Parent parentState, ReduxAction action) {
    ClassType state = getter(parentState);

    Iterable<ActionReducer> result =
        reducers.where((element) => element.action == action.runtimeType);

    ActionReducer? actionReducer = result.isNotEmpty ? result.first : null;

    if (actionReducer != null) {
      return setter(parentState, actionReducer.call(state, action.value));
    }

    for (ReduxReducer<ClassType, dynamic> childReducer in children) {
      ClassType? newChildState = childReducer.reduce(state, action);

      if (newChildState != null) {
        return setter(parentState, newChildState);
      }
    }
  }
}

==> 1.0.3.tar.gz: lib/redux_reducer.dart <==
import 'package:redux_enhancement/action_reducer.dart';
import 'package:redux_enhancement/redux_action.dart';
import 'package:redux_enhancement/redux_types.dart';

/// Every Reducer has to extend this class
/// ```
/// class ShopReducer extends ReduxReducer<AppState, Shop> {
///   static Map<Type, ReducerFunction<AppState, dynamic>> _reducers = {
///     UpdateShopNameAction: (shop, action) => shop..name = action.value,
///   }
///
///   ShopReducer()
///       : super(
///     getter: (appState) => appState.shop,
///     setter: (appState, value) => appState..shop = value,
///     reducers: _reducers,
///     children: [
///       AddressReducer<Shop>(
///             (shop) => shop.address,
///             (shop, address) => shop..address = address,
///       ),
///     ],
///   );
/// }
/// ```
///
abstract class ReduxReducer<Parent, ClassType> {
  GetterFunction<Parent, ClassType> getter;

  SetterFunction<Parent, ClassType> setter;

  List<ReduxReducer<ClassType, dynamic>> children;

  List<ActionReducer<ClassType, dynamic, dynamic>> reducers;

  ReduxReducer({
    required this.getter,
    required this.setter,
    required this.reducers,
    this.children = const [],
  });

  /// Retrieves the fitting reducer for the [ReduxAction] supplied, executes it and returns the [Parent]
  /// or null if no [ReducerFunction] was found in its [reducers] or within its [children]
  /// The first [ReducerFunction] that is found is used and the result returned immediately.
  /// You should never have multiple [ReducerFunction]s defined for any [ReduxAction]!
  Parent? reduce(Parent parentState, ReduxAction action) {
    ClassType state = getter(parentState);

    Iterable<ActionReducer> result = reducers.where((element) => element.action == action.runtimeType);

    ActionReducer? actionReducer = result.isNotEmpty ? result.first : null;

    if (actionReducer != null) {
      return setter(parentState, actionReducer.call(state, action.value));
    }

    for (ReduxReducer<ClassType, dynamic> childReducer in children) {
      ClassType? newChildState = childReducer.reduce(state, action);

      if (newChildState != null) {
        return setter(parentState, newChildState);
      }
    }
  }
}
